# Generated by roxygen2: do not edit by hand

S3method(ClusteringEntropy,"NULL")
S3method(ClusteringEntropy,Splits)
S3method(ClusteringEntropy,list)
S3method(ClusteringEntropy,multiPhylo)
S3method(ClusteringEntropy,phylo)
S3method(ClusteringInfo,"NULL")
S3method(ClusteringInfo,Splits)
S3method(ClusteringInfo,list)
S3method(ClusteringInfo,multiPhylo)
S3method(ClusteringInfo,phylo)
S3method(DistanceFromMedian,dist)
S3method(DistanceFromMedian,numeric)
S3method(KCDiameter,list)
S3method(KCDiameter,multiPhylo)
S3method(KCDiameter,numeric)
S3method(KCDiameter,phylo)
S3method(KMeansPP,dist)
S3method(KMeansPP,matrix)
S3method(KMeansPP,numeric)
S3method(MeanMSTEdge,dist)
S3method(MeanMSTEdge,numeric)
S3method(MeanNN,dist)
S3method(MeanNN,numeric)
S3method(NNIDiameter,list)
S3method(NNIDiameter,multiPhylo)
S3method(NNIDiameter,numeric)
S3method(NNIDiameter,phylo)
S3method(NTip,HPart)
S3method(RenumberTips,HPart)
S3method(SPRDist,list)
S3method(SPRDist,multiPhylo)
S3method(SPRDist,phylo)
S3method(SplitwiseInfo,"NULL")
S3method(SplitwiseInfo,Splits)
S3method(SplitwiseInfo,list)
S3method(SplitwiseInfo,multiPhylo)
S3method(SplitwiseInfo,phylo)
S3method(as.HPart,HPart)
S3method(as.HPart,default)
S3method(as.HPart,list)
S3method(as.HPart,phylo)
S3method(as.phylo,HPart)
S3method(clone,HPart)
S3method(median,multiPhylo)
S3method(plot,HPart)
S3method(print,HPart)
export(.ThreeDPlotServer)
export(.TreeDistance)
export(AHMI)
export(AllSplitPairings)
export(CalculateTreeDistance)
export(ClusteringEntropy)
export(ClusteringInfo)
export(ClusteringInfoDist)
export(ClusteringInfoDistance)
export(CompareAll)
export(ConsensusInfo)
export(DifferentPhylogeneticInfo)
export(DisplayMatching)
export(DistFromMed)
export(DistanceFromMedian)
export(EHMI)
export(Entropy)
export(ExpectedVariation)
export(GeneralizedRF)
export(GetParallel)
export(HH)
export(HMI)
export(HierarchicalMutualInfo)
export(HierarchicalMutualInformation)
export(InfoRobinsonFoulds)
export(InfoRobinsonFouldsSplits)
export(Islands)
export(JaccardRobinsonFoulds)
export(JaccardSplitSimilarity)
export(KCDiameter)
export(KCVector)
export(KMeansPP)
export(KendallColijn)
export(LAPJV)
export(LnTreesConsistentWithTwoSplits)
export(Log2TreesConsistentWithTwoSplits)
export(MASTInfo)
export(MASTSize)
export(MCITree)
export(MSTSegments)
export(MapTrees)
export(MappingQuality)
export(MatchingSplitDistance)
export(MatchingSplitDistanceSplits)
export(MatchingSplitInfo)
export(MatchingSplitInfoDistance)
export(MatchingSplitInfoSplits)
export(MeanCentDist)
export(MeanCentroidDist)
export(MeanCentroidDistance)
export(MeanMSTEdge)
export(MeanNN)
export(MeilaMutualInformation)
export(MeilaVariationOfInformation)
export(MutualClusteringInfo)
export(MutualClusteringInfoSplits)
export(MutualClusteringInformation)
export(NNIDiameter)
export(NNIDist)
export(NormalizeInfo)
export(Ntropy)
export(NyeSimilarity)
export(NyeSplitSimilarity)
export(PathDist)
export(PathVector)
export(PhylogeneticInfoDistance)
export(Plot3)
export(PlotMatching)
export(Project)
export(ProjectionQuality)
export(ReduceTrees)
export(ReportMatching)
export(RobinsonFoulds)
export(RobinsonFouldsInfo)
export(RobinsonFouldsMatching)
export(RobinsonFouldsSplits)
export(SPRDist)
export(SelfHMI)
export(SetParallel)
export(SharedPhylogeneticInfo)
export(SharedPhylogeneticInfoSplits)
export(SpectralClustering)
export(SpectralEigens)
export(SplitDifferentInformation)
export(SplitEntropy)
export(SplitSharedInformation)
export(SplitVector)
export(SplitsCompatible)
export(SplitwiseInfo)
export(StartParallel)
export(StopParallel)
export(StrainCol)
export(SumOfRanges)
export(SumOfVariances)
export(SumOfVars)
export(TransferConsensus)
export(TransferDist)
export(TransferDistSplits)
export(TransferDistance)
export(TreeDistPlot)
export(TreeDistance)
export(TreesConsistentWithTwoSplits)
export(VisualiseMatching)
export(VisualizeMatching)
export(as.HPart)
export(clone)
export(entropy_int)
export(is.HPart)
importFrom(Rdpack,reprompt)
importFrom(TreeTools,AllAncestors)
importFrom(TreeTools,Consensus)
importFrom(TreeTools,DropTip)
importFrom(TreeTools,FirstMatchingSplit)
importFrom(TreeTools,KeepTip)
importFrom(TreeTools,KeepTipPostorder)
importFrom(TreeTools,LnRooted.int)
importFrom(TreeTools,LnTreesMatchingSplit)
importFrom(TreeTools,Log2Rooted.int)
importFrom(TreeTools,Log2TreesMatchingSplit)
importFrom(TreeTools,Log2Unrooted)
importFrom(TreeTools,Log2Unrooted.int)
importFrom(TreeTools,MSTEdges)
importFrom(TreeTools,MSTLength)
importFrom(TreeTools,MatchStrings)
importFrom(TreeTools,NRooted)
importFrom(TreeTools,NSplits)
importFrom(TreeTools,NTip)
importFrom(TreeTools,PairwiseDistances)
importFrom(TreeTools,PectinateTree)
importFrom(TreeTools,Postorder)
importFrom(TreeTools,PostorderOrder)
importFrom(TreeTools,Preorder)
importFrom(TreeTools,RenumberTips)
importFrom(TreeTools,RootOnNode)
importFrom(TreeTools,RootTree)
importFrom(TreeTools,SplitFrequency)
importFrom(TreeTools,SplitInformation)
importFrom(TreeTools,SplitsInBinaryTree)
importFrom(TreeTools,StarTree)
importFrom(TreeTools,TipLabels)
importFrom(TreeTools,TipsInSplits)
importFrom(TreeTools,TreeIsRooted)
importFrom(TreeTools,TreesMatchingSplit)
importFrom(TreeTools,as.ClusterTable)
importFrom(TreeTools,as.Splits)
importFrom(TreeTools,edge_to_splits)
importFrom(ape,Nnode.phylo)
importFrom(ape,as.phylo)
importFrom(ape,drop.tip)
importFrom(ape,edgelabels)
importFrom(ape,nodelabels)
importFrom(ape,plot.phylo)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_progress_along)
importFrom(cli,cli_progress_bar)
importFrom(cli,cli_progress_update)
importFrom(colorspace,qualitative_hcl)
importFrom(colorspace,sequential_hcl)
importFrom(grDevices,colorRamp)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rgb)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(parallel,makeCluster)
importFrom(parallel,parCapply)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(shiny,runApp)
importFrom(shinyjs,useShinyjs)
importFrom(stats,as.dist)
importFrom(stats,dist)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(utils,install.packages)
useDynLib(TreeDist, .registration = TRUE)
