% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree_distance_info.R
\name{.PairMean}
\alias{.PairMean}
\title{Mean of two numbers}
\usage{
.PairMean(x, y)
}
\description{
Used for normalization and range calculation
}
\keyword{internal}
