% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hierarchical_mutual_information.R
\name{HH}
\alias{HH}
\title{Self hierarchical mutual information}
\usage{
HH(tree)
}
\description{
An alias of \code{SelfHMI()}
}
\keyword{internal}
