% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree_distance_utilities.R
\name{ReportMatching}
\alias{ReportMatching}
\title{List clades as text}
\usage{
ReportMatching(splits1, splits2, realMatch = TRUE)
}
\arguments{
\item{splits1, splits2}{Logical matrices with columns specifying membership
of each corresponding matched clade.}
}
\value{
\code{ReportMatching} returns a character vector describing each pairing
in a matching.
}
\description{
List clades as text
}
\seealso{
\code{\link{VisualizeMatching}}
}
\author{
\href{https://orcid.org/0000-0001-5660-1727}{Martin R. Smith}
(\href{mailto:martin.smith@durham.ac.uk}{martin.smith@durham.ac.uk})
}
\keyword{internal}
